package betterquesting.api2.client.gui.panels.lists;

import betterquesting.api.utils.RenderUtils;
import betterquesting.api2.client.gui.controls.IValueIO;
import betterquesting.api2.client.gui.controls.io.FloatSimpleIO;
import betterquesting.api2.client.gui.misc.ComparatorGuiDepth;
import betterquesting.api2.client.gui.misc.GuiRectangle;
import betterquesting.api2.client.gui.misc.IGuiRect;
import betterquesting.api2.client.gui.panels.IGuiCanvas;
import betterquesting.api2.client.gui.panels.IGuiPanel;
import java.util.List;
import java.util.ListIterator;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.annotation.Nonnull;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL11;

/* loaded from: betterquesting_api2_client_gui_panels_lists.jar:betterquesting/api2/client/gui/panels/lists/CanvasScrolling.class */
public class CanvasScrolling implements IGuiCanvas {
    private final IGuiRect transform;
    private final List<IGuiPanel> guiPanels = new CopyOnWriteArrayList();
    private boolean enabled = true;
    protected final GuiRectangle scrollBounds = new GuiRectangle(0, 0, 0, 0);
    private final GuiRectangle scrollWindow = new GuiRectangle(0, 0, 0, 0);
    protected boolean extendedScroll = false;
    protected boolean zoomMode = false;
    protected int margin = 0;
    private boolean isDragging = false;
    private boolean hasDragged = false;
    protected int scrollSpeed = 12;
    private float dragSX = 0.0f;
    private float dragSY = 0.0f;
    private int dragMX = 0;
    private int dragMY = 0;
    protected float lsz = 1.0f;
    protected int lsx = 0;
    protected int lsy = 0;
    private boolean useBlocking = true;
    private final CanvasCullingManager cullingManager = new CanvasCullingManager();
    protected IValueIO<Float> scrollX = new FloatSimpleIO().setLerp(true, 0.02f);
    protected IValueIO<Float> scrollY = new FloatSimpleIO().setLerp(true, 0.02f);
    protected IValueIO<Float> zoomScale = new FloatSimpleIO(1.0f, 0.2f, 2.0f).setLerp(true, 0.02f);

    public CanvasScrolling(IGuiRect rect) {
        this.transform = rect;
    }

    public CanvasScrolling setScrollDriverX(IValueIO<Float> driver) {
        this.scrollX = driver;
        return this;
    }

    public CanvasScrolling setScrollDriverY(IValueIO<Float> driver) {
        this.scrollY = driver;
        return this;
    }

    public CanvasScrolling setZoomDriver(IValueIO<Float> driver) {
        this.zoomScale = driver;
        return this;
    }

    public CanvasScrolling setScrollSpeed(int dx) {
        this.scrollSpeed = dx;
        return this;
    }

    public CanvasScrolling setupAdvanceScroll(boolean scrollToZoom, boolean extendedScroll, int scrollMargin) {
        this.zoomMode = scrollToZoom;
        this.extendedScroll = extendedScroll;
        this.margin = scrollMargin;
        return this;
    }

    public CanvasScrolling enableBlocking(boolean state) {
        this.useBlocking = state;
        return this;
    }

    public IGuiRect getScrollBounds() {
        return this.scrollBounds;
    }

    public int getScrollX() {
        return Math.round(this.scrollBounds.getX() + (this.scrollBounds.getWidth() * ((Float) this.scrollX.readValue()).floatValue()));
    }

    public int getScrollY() {
        return Math.round(this.scrollBounds.getY() + (this.scrollBounds.getHeight() * ((Float) this.scrollY.readValue()).floatValue()));
    }

    public float getZoom() {
        return ((Float) this.zoomScale.readValue()).floatValue();
    }

    public void setScrollX(int sx) {
        if (this.scrollBounds.getWidth() <= 0) {
            return;
        }
        this.scrollX.writeValueRaw(Float.valueOf((sx - this.scrollBounds.getX()) / this.scrollBounds.getWidth()));
        this.lsx = getScrollX();
    }

    public void setScrollY(int sy) {
        if (this.scrollBounds.getHeight() <= 0) {
            return;
        }
        this.scrollY.writeValueRaw(Float.valueOf((sy - this.scrollBounds.getY()) / this.scrollBounds.getHeight()));
        this.lsy = getScrollY();
    }

    public void setZoom(float z) {
        this.zoomScale.writeValueRaw(Float.valueOf(z));
        this.lsz = ((Float) this.zoomScale.readValue()).floatValue();
        refreshScrollBounds();
    }

    public void initPanel() {
        this.guiPanels.clear();
        this.cullingManager.reset();
    }

    public void setEnabled(boolean state) {
        this.enabled = state;
    }

    public boolean isEnabled() {
        return this.enabled;
    }

    public IGuiRect getTransform() {
        return this.transform;
    }

    @Nonnull
    public List<IGuiPanel> getChildren() {
        return this.guiPanels;
    }

    public void drawPanel(int mx, int my, float partialTick) {
        float zs = ((Float) this.zoomScale.readValue()).floatValue();
        int tx = this.transform.getX();
        int ty = this.transform.getY();
        if (this.isDragging && (Mouse.isButtonDown(0) || Mouse.isButtonDown(2))) {
            int dx = (int) ((this.dragMX - mx) / zs);
            int dy = (int) ((this.dragMY - my) / zs);
            if (this.scrollBounds.getWidth() > 0) {
                float dsx = (dx / this.scrollBounds.getWidth()) + this.dragSX;
                this.scrollX.writeValue(Float.valueOf(dsx));
                if (!this.hasDragged && Math.abs(this.dragSX - ((Float) this.scrollX.readValue()).floatValue()) > 0.05f) {
                    this.hasDragged = true;
                }
            }
            if (this.scrollBounds.getHeight() > 0) {
                float dsy = (dy / this.scrollBounds.getHeight()) + this.dragSY;
                this.scrollY.writeValue(Float.valueOf(dsy));
                if (!this.hasDragged && Math.abs(this.dragSY - ((Float) this.scrollY.readValue()).floatValue()) > 0.05f) {
                    this.hasDragged = true;
                }
            }
        } else if (this.isDragging || this.hasDragged) {
            this.isDragging = false;
            this.hasDragged = false;
        }
        if (this.lsz != zs) {
            float change = zs / this.lsz;
            int csx = getScrollX();
            int csy = getScrollY();
            float swcx = this.scrollWindow.w / 2.0f;
            float swcy = this.scrollWindow.h / 2.0f;
            float swcx2 = swcx - (swcx / change);
            float swcy2 = swcy - (swcy / change);
            refreshScrollBounds();
            if (this.scrollBounds.getWidth() > 0) {
                this.scrollX.writeValue(Float.valueOf(((csx + swcx2) - this.scrollBounds.getX()) / this.scrollBounds.getWidth()));
            }
            if (this.scrollBounds.getHeight() > 0) {
                this.scrollY.writeValue(Float.valueOf(((csy + swcy2) - this.scrollBounds.getY()) / this.scrollBounds.getHeight()));
            }
            this.lsx = getScrollX();
            this.lsy = getScrollY();
            this.lsz = zs;
        } else if (this.lsx != getScrollX() || this.lsy != getScrollY()) {
            updatePanelScroll();
        }
        GL11.glPushMatrix();
        RenderUtils.startScissor(this.transform);
        GL11.glTranslatef(tx - (this.lsx * zs), ty - (this.lsy * zs), 0.0f);
        GL11.glScalef(zs, zs, zs);
        int smx = ((int) ((mx - tx) / zs)) + this.lsx;
        int smy = ((int) ((my - ty) / zs)) + this.lsy;
        for (IGuiPanel panel : getVisiblePanels()) {
            if (panel.isEnabled()) {
                panel.drawPanel(smx, smy, partialTick);
            }
        }
        RenderUtils.endScissor();
        GL11.glPopMatrix();
    }

    public boolean onMouseClick(int mx, int my, int click) {
        if (!this.transform.contains(mx, my)) {
            return false;
        }
        float zs = ((Float) this.zoomScale.readValue()).floatValue();
        int tx = this.transform.getX();
        int ty = this.transform.getY();
        int smx = ((int) ((mx - tx) / zs)) + this.lsx;
        int smy = ((int) ((my - ty) / zs)) + this.lsy;
        boolean used = false;
        ListIterator<IGuiPanel> pnIter = this.guiPanels.listIterator(this.guiPanels.size());
        while (true) {
            if (!pnIter.hasPrevious()) {
                break;
            }
            IGuiPanel entry = pnIter.previous();
            if (entry.isEnabled() && entry.onMouseClick(smx, smy, click)) {
                used = true;
                break;
            }
        }
        if (!used && (click == 0 || click == 2)) {
            this.dragSX = ((Float) this.scrollX.readValue()).floatValue();
            this.dragSY = ((Float) this.scrollY.readValue()).floatValue();
            this.dragMX = mx;
            this.dragMY = my;
            this.isDragging = true;
            return true;
        }
        return used;
    }

    public boolean onMouseRelease(int mx, int my, int click) {
        boolean used = false;
        if (!this.hasDragged) {
            if (!this.transform.contains(mx, my)) {
                return false;
            }
            float zs = ((Float) this.zoomScale.readValue()).floatValue();
            int tx = this.transform.getX();
            int ty = this.transform.getY();
            int smx = ((int) ((mx - tx) / zs)) + this.lsx;
            int smy = ((int) ((my - ty) / zs)) + this.lsy;
            ListIterator<IGuiPanel> pnIter = this.guiPanels.listIterator(this.guiPanels.size());
            while (true) {
                if (!pnIter.hasPrevious()) {
                    break;
                }
                if (pnIter.previous().onMouseRelease(smx, smy, click)) {
                    used = true;
                    break;
                }
            }
        }
        if (this.isDragging) {
            if (Mouse.isButtonDown(0) || Mouse.isButtonDown(2)) {
                return true;
            }
            this.isDragging = false;
            return true;
        }
        return used;
    }

    public boolean onMouseScroll(int mx, int my, int scroll) {
        if (scroll == 0 || !this.transform.contains(mx, my)) {
            return false;
        }
        float zs = ((Float) this.zoomScale.readValue()).floatValue();
        int tx = this.transform.getX();
        int ty = this.transform.getY();
        int smx = ((int) ((mx - tx) / zs)) + this.lsx;
        int smy = ((int) ((my - ty) / zs)) + this.lsy;
        boolean used = false;
        ListIterator<IGuiPanel> pnIter = this.guiPanels.listIterator(this.guiPanels.size());
        while (true) {
            if (!pnIter.hasPrevious()) {
                break;
            }
            IGuiPanel entry = pnIter.previous();
            if (entry.isEnabled() && entry.onMouseScroll(smx, smy, scroll)) {
                used = true;
                break;
            }
        }
        if (!used) {
            if (this.zoomMode) {
                this.zoomScale.writeValue(Float.valueOf(((Float) this.zoomScale.readValueRaw()).floatValue() + ((-scroll) * 0.05f)));
            } else if (this.scrollBounds.getHeight() > 0) {
                float dy = (scroll * this.scrollSpeed) / this.scrollBounds.getHeight();
                float cs = ((Float) this.scrollY.readValue()).floatValue();
                if ((dy >= 0.0f || cs > 0.0f) && (dy <= 0.0f || cs < 1.0f)) {
                    this.scrollY.writeValue(Float.valueOf(cs + dy));
                    updatePanelScroll();
                }
            }
        }
        return used;
    }

    public boolean onKeyTyped(char c, int keycode) {
        boolean used = false;
        ListIterator<IGuiPanel> pnIter = this.guiPanels.listIterator(this.guiPanels.size());
        while (true) {
            if (!pnIter.hasPrevious()) {
                break;
            }
            IGuiPanel entry = pnIter.previous();
            if (entry.isEnabled() && entry.onKeyTyped(c, keycode)) {
                used = true;
                break;
            }
        }
        if (!used && c == 'c') {
            setScrollX(0);
            setScrollY(0);
        }
        return used;
    }

    public List<String> getTooltip(int mx, int my) {
        List<String> tt;
        if (!this.transform.contains(mx, my) || this.isDragging) {
            return null;
        }
        float zs = ((Float) this.zoomScale.readValue()).floatValue();
        int tx = this.transform.getX();
        int ty = this.transform.getY();
        int smx = ((int) ((mx - tx) / zs)) + this.lsx;
        int smy = ((int) ((my - ty) / zs)) + this.lsy;
        List<IGuiPanel> tmpList = getVisiblePanels();
        ListIterator<IGuiPanel> pnIter = tmpList.listIterator(tmpList.size());
        while (pnIter.hasPrevious()) {
            IGuiPanel entry = pnIter.previous();
            if (entry.isEnabled() && (tt = entry.getTooltip(smx, smy)) != null && tt.size() > 0) {
                return tt;
            }
        }
        return null;
    }

    public void addPanel(IGuiPanel panel) {
        addCulledPanel(panel, true);
    }

    public void addCulledPanel(IGuiPanel panel, boolean useCulling) {
        if (panel == null || this.guiPanels.contains(panel)) {
            return;
        }
        this.guiPanels.add(panel);
        this.guiPanels.sort(ComparatorGuiDepth.INSTANCE);
        this.cullingManager.addPanel(panel, useCulling);
        panel.initPanel();
        refreshScrollBounds();
    }

    public boolean removePanel(IGuiPanel panel) {
        boolean b = this.guiPanels.remove(panel);
        if (b) {
            this.cullingManager.removePanel(panel);
            refreshScrollBounds();
        }
        return b;
    }

    public void refreshScrollBounds() {
        boolean first = true;
        int left = 0;
        int right = 0;
        int top = 0;
        int bottom = 0;
        float zs = ((Float) this.zoomScale.readValue()).floatValue();
        for (IGuiPanel panel : this.guiPanels) {
            if (first) {
                left = panel.getTransform().getX();
                top = panel.getTransform().getY();
                right = panel.getTransform().getX() + panel.getTransform().getWidth();
                bottom = panel.getTransform().getY() + panel.getTransform().getHeight();
                first = false;
            } else {
                left = Math.min(left, panel.getTransform().getX());
                top = Math.min(top, panel.getTransform().getY());
                right = Math.max(right, panel.getTransform().getX() + panel.getTransform().getWidth());
                bottom = Math.max(bottom, panel.getTransform().getY() + panel.getTransform().getHeight());
            }
        }
        int left2 = left - this.margin;
        int right2 = right + this.margin;
        int top2 = top - this.margin;
        int bottom2 = bottom + this.margin;
        int right3 = right2 - ((int) Math.ceil(getTransform().getWidth() / zs));
        int bottom3 = bottom2 - ((int) Math.ceil(getTransform().getHeight() / zs));
        if (this.extendedScroll) {
            this.scrollBounds.x = Math.min(left2, right3);
            this.scrollBounds.y = Math.min(top2, bottom3);
            this.scrollBounds.w = Math.max(left2, right3) - this.scrollBounds.x;
            this.scrollBounds.h = Math.max(top2, bottom3) - this.scrollBounds.y;
        } else {
            this.scrollBounds.x = left2;
            this.scrollBounds.y = top2;
            this.scrollBounds.w = Math.max(0, right3 - left2);
            this.scrollBounds.h = Math.max(0, bottom3 - top2);
        }
        updatePanelScroll();
    }

    public void updatePanelScroll() {
        this.lsx = getScrollX();
        this.lsy = getScrollY();
        float zs = ((Float) this.zoomScale.readValue()).floatValue();
        this.scrollWindow.x = this.lsx;
        this.scrollWindow.y = this.lsy;
        this.scrollWindow.w = (int) Math.ceil(this.transform.getWidth() / zs);
        this.scrollWindow.h = (int) Math.ceil(this.transform.getHeight() / zs);
        this.cullingManager.updateVisiblePanels(this.scrollWindow);
    }

    public void resetCanvas() {
        this.guiPanels.clear();
        this.cullingManager.reset();
        refreshScrollBounds();
    }

    private List<IGuiPanel> getVisiblePanels() {
        return this.useBlocking ? this.cullingManager.getVisiblePanels() : this.guiPanels;
    }
}
